#!/usr/bin/env Rscript

# For more information, please see https://plot.ly/r/shinyapp-explore-diamonds/
library(configr)
library(ggplot2)
library(plotly)
library(dplyr)
library(ggcorrplot)
library(RColorBrewer)

read_json <- function(jsonfile) {
  return(read.config(jsonfile))
}

getColors <- function(n, dataType = "seq") {
  allChoices <- brewer.pal.info

  availableChoices <- allChoices[allChoices$maxcolors > n & allChoices$category == dataType,]

  if (dim(availableChoices)[1] != 0) {
    colorPal <- sample(rownames(availableChoices), size = 1)
    if (n <= 3) {
      options <- brewer.pal(3, colorPal)
      return(options[1:n])
    } else {
      return(brewer.pal(n, colorPal))
    }
  } else {
    return(NULL)
  }
}

getLen <- function(x) {
  return(length(unique(x)))
}

make_corrplot <- function(args_json) {
  input <- read_json(args_json)
  data <- read.csv(input$datafile)
  colors <- getColors(3, "div")

  result <- cor(data[, input$corr_vars])
  # build graph with ggplot syntax
  p <- ggcorrplot(result, method = input$method, type = input$corr_type,
                      show.diag = TRUE, colors = colors,
                      outline.col = "white", hc.order = input$hc_order,
                      hc.method = input$hc_method, lab = TRUE,
                      lab_col = "black", lab_size = 4, p.mat = NULL,
                      sig.level = input$sig_level, title = input$title,
                      show.legend = TRUE, legend.title = "", insig = "pch",
                      ggtheme = ggplot2::theme_gray, pch = 4, pch.col = "black") +
      theme(axis.text.x = element_text(angle = 0, hjust = 1, margin = margin(-3, 0, 0, 0),
                                              size = 11),
                      axis.text.y = element_text(size = 11,
                                              margin = margin(0, -3, 0, 0)),
                      text = element_text(size = input$corrplot_title_size),
                      legend.text = element_text(size = 11),
                      legend.title = element_blank(),
                      panel.background = element_blank(),
                      panel.grid.major = element_blank(),
                      axis.line = element_line(colour = "black"))

  return(ggplotly(p))
}

write_json <- function(plotly_object, output_file) {
  json <- plotly::plotly_json(plotly_object, FALSE)
  write(json, output_file)
}

corrplot <- function(args_json, output_file) {
  plotly_object <- make_corrplot(args_json)
  write_json(plotly_object, output_file)
}
